/*
 * Copyright (C) 1999/2000 Sebastian Krahmer.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Sebastian Krahmer.
 * 4. The name Sebastian Krahmer may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*** Accounting class and some other functions that
 *** belong to it.
 ***/
#include <stdio.h>
#include <sys/types.h>
#include <sys/acct.h>
#include <pwd.h>
#include <grp.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <iostream>
#include <errno.h>
#include "config.h"
#include "acct.h"

int printAcct(struct acct);
uid_t user2uid(char *);
gid_t group2gid(char *);

extern "C" {
void endgrent(void);
void endpwent(void);
}

accounting::accounting(const char *filename)
{
	memset(&ac, 0, sizeof(ac));
	memset(file, 0 , MAXPATHLEN);
	strncpy(file, filename, MAXPATHLEN);
        
        if ((acct_fd = open(filename, O_RDONLY)) < 0) {
           	perror("open");
                exit(errno);
        }
}

accounting::~accounting()
{
   	close(acct_fd);
}

/* Show accounting-information of user with UID uid,
 * or all if uid == -1.
 */
int accounting::show_uid(uid_t uid)
{
   	
        while (read(acct_fd, &ac, sizeof(ac)) > 0) {
           	if (uid == (uid_t)-1 || ac.ac_uid == uid)
                   	printAcct(ac);
		memset(&ac, 0, sizeof(ac));
        }
	return 0;
}

int accounting::show_user(char *user)
{
   	return show_uid(user2uid(user));
}

/* Show  accounting-information of all users that belogng to
 * group with GID gid, or all groups if gid == -1.
 */
int accounting::show_gid(gid_t gid)
{
   	while (read(acct_fd, &ac, sizeof(ac)) > 0) {
           	if (gid == (gid_t)-1 || ac.ac_uid == gid)
                   	printAcct(ac);
		memset(&ac, 0, sizeof(ac));
        }
        return 0;
}
	
int accounting::show_group(char *group)
{
   	return show_gid(group2gid(group));
}
    

/* Map a username to its UID.
 */
uid_t user2uid(char *user)
{
	struct passwd *pw;
	uid_t r;
	
	if ((pw = getpwnam(user)) == NULL) {
		perror("getpwnam");
		exit(errno);
	}
	r = pw->pw_uid;
	endpwent();
	return r;
}

/* Map a groupname to its GID.
 */	
uid_t group2gid(char *group)
{
   	struct group *ge;
        gid_t r;
        
        if ((ge = getgrnam((char*)group)) == NULL) {
           	perror("getgrpnam");
                exit(errno);
        }
        r = ge->gr_gid;
        endgrent();
        return r;
}
        
 
/* Print-routine for a struct acct, where i think it's good.
 */ 
int printAcct(struct acct a)
{
   	const time_t b_time = a.ac_btime;
	char *b_date = ctime(&b_time);
	
	
	/* skip terminating newline if one */
	if (b_date[strlen(b_date) - 1] == '\n')
		b_date[strlen(b_date) - 1] = '\0';


	printf("%s: ", b_date);

	printf(" (UID = %d, GID = %d) tty: %d '", a.ac_uid, a.ac_gid, a.ac_tty);
	
	/* print out executed program */
	for (unsigned int i = 0; i < sizeof(a.ac_comm); i++) {
		if (isprint(a.ac_comm[i]))
			printf("%c", a.ac_comm[i]);
	}
	printf("'\n");
        return 0;
}
        
/* Some functions that doesn't belong to the accounting-class.
 */

int enable_accounting(const char *file)
{

	cout<<"Enable process-accounting via file "<<file<<".\n";
	
	if (access(file, W_OK) != 0) {
		creat(file, 0600);
	}

	if (acct(file) < 0) {
		perror("acct");
		exit(errno);
	}
	return 0;
}

int disable_accounting()
{
	cout<<"Disabling process-accounting.\n";
	if (acct(NULL) < 0) {
		perror("acct");
		exit(errno);
	}
	return 0;
}

