/*
 * Copyright (C) 1999/2000 Sebastian Krahmer.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Sebastian Krahmer.
 * 4. The name Sebastian Krahmer may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#include <stdio.h>
#include <iostream>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include "config.h"
#include "exec.h"

uid_t user2uid(char *);
gid_t group2gid(char *);

 
#ifdef HAVE_EOE
#include <linux/eoe.h>

void installEoE()
{
   	system("insmod /lib/modules/eoe.o");
}


/* Constructor: open device and try to install module if 
 * it fails.
 */
EoE::EoE(const char *filename) throw(EoEexception)
{
   	
	if ((fd = open(filename, O_RDONLY)) < 0) {
           	installEoE();
                
                /* if this fails, the installation of EoE failed.
                 */
                if ((fd = open(filename, O_RDONLY)) < 0) 
                   	throw EoEexception(errno);
        }
}

EoE::~EoE()
{
	close(fd);
}

/* read one line from the eoe-device, at most
 * 'len' bytes.
 */
int EoE::read(void *buf, int len)
{
	int r = 0;

	memset(buf, 0, len);
	if ((r = ::read(fd, buf, len)) < 0) {
		perror("read");
		exit(errno);
	}
	return r;
}


/* set the euid that should be monitored, -1 if all euids
 * must be monitored
 */
int EoE::setMonitorEuid(uid_t euid)
{
	if (euid == -1) {
		if (ioctl(fd, EOE_SETALL) < 0) {
			perror("ioctl (EOE_SETALL)");
			exit(errno);
		}
	} else {
		if (ioctl(fd, EOE_SETEUID, &euid) < 0) {
			perror("ioctl (EOE_SETEUID)");
			exit(errno);
		}
	}
	return 0;
}

#else /* HAVE_EOE */

EoE::EoE(const char *filename) throw(EoEexception)
{
   	cerr<<"Warning: Not configured to use EoE! Please install EoE and compile\n"
              "         and install me again. (exiting)\n\n";
	exit(1);
}

EoE::~EoE()
{
}

/* 
 */
int EoE::read(void *buf, int len)
{
   	return 0;
}


/* 
 */
int EoE::setMonitorEuid(uid_t euid)
{
   	return 0;
}

#endif /* HAVE_EOE */

/* Log all programs to strandart logfiles
 * that 'euid' executes.
 */
int EoElog(uid_t euid)
{
   	EoE *E;
        
   	try {
           	E = new EoE;
        } catch (EoEexception e) {
           	e.print();
                exit(1);
        }
        int pid = fork();
        
        /* in parent case. return */
        if (pid > 0) {
                return 0;
                
        /* in child-case log */
        } else if (pid == 0) {
		int r = 0, j = 0;
		
           	E->setMonitorEuid(euid);
           	char *logbuf = new char[1000];
		char *cleanlog = new char[4000];
                
           	openlog("EoE", LOG_PID, LOG_AUTH);
           	while (1) {
                   	r = E->read(logbuf, 1000);
			
			/* escape all % in logbuf to avoid SEGFAULTs */
			for (int i = 0, j = 0; i < r && j < 4000; i++) {
				if (logbuf[i] == '%') {
					cleanlog[j++] = '?'; //'\\';				
				} else 
					cleanlog[j++] = logbuf[i];
			}
                        syslog(LOG_NOTICE, cleanlog);
                        memset(logbuf, 0, 1000);
			memset(cleanlog, 0, 4000);
                }
                   	
        } 
 
        /* if control goes here, sth. must went wrong
         */
        perror("fork");
        exit(errno);
}

int EoElogUser(char *username)
{
   	return EoElog(user2uid(username));
}

        
