/*
 * Copyright (C) 1999/2000 Sebastian Krahmer.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Sebastian Krahmer.
 * 4. The name Sebastian Krahmer may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#include <sys/types.h>
#include <stdio.h>
#include <popt.h>
#include <iostream>
#include "acct.h"

int put_banner();
void help_her();

extern int enable_accounting(const char *);
extern int disable_accounting();
extern int EoElog(uid_t);
extern int EoElogUser(char*);

const char *const defaultAccFile = "/var/log/acct";

int main(int argc, char *argv[])
{
   	int disable = 0, enable = 0, uid = -1, 
	    gid = -1, all = 0, eoe_euid = -1;
        char *user = NULL, *group = NULL, *eoe_user = NULL;
        const char *AccFile = NULL;
	int c = 0;
        
   	poptContext pc;
        struct poptOption optTable[] = {
           	{"disable", 'd', POPT_ARG_NONE, &disable, 1,
                 "disable process-accounting", NULL
                },
                {"enable", 'e', POPT_ARG_NONE, &enable, 1,
                 "enable process-acconting", NULL
                },
                {"uid", 'u', POPT_ARG_INT, &uid, 1,
                 "show information about UID", "uid"
                },
                {"gid", 'g', POPT_ARG_INT, &gid, 1,
                 "show information about GID", "gid"
                },
                {"user", 'U', POPT_ARG_STRING, &user, 1,
                 "same as 'u' but with userNAME", "user"
                },
                {"group", 'G', POPT_ARG_STRING, &group, 1,
                 "same as 'g' but with groupNAME", "group"
                },
                {"file", 'f', POPT_ARG_STRING, &AccFile, 1,
                 "specify accounting-file, default is /var/log/acct", "file"
                },
		{"all", 'a', POPT_ARG_NONE, &all, 1,
		 "show infos from all users", NULL
		},
		{"log-euid", 'l', POPT_ARG_INT, &eoe_euid, 1,
		 "EoE: log what EUID 'euid' does.", "euid",
		},
		{"log-user", 'L', POPT_ARG_STRING, &eoe_user, 1,
		 "EoE: log what user 'user' does.", "user",
		},
                POPT_AUTOHELP
                {NULL, 0, 0, NULL, 0, NULL, NULL
                }
        };
        
        put_banner();
	if (argc < 2)
		help_her();
        
        pc = poptGetContext(NULL, argc, argv, optTable, 0);
        
        while ((c = poptGetNextOpt(pc)) >= 0)
           	;
                
        if (c < -1)
           	help_her();
        
        /* if no accounting-file was specified
         */
        if (AccFile == NULL)
           	AccFile = defaultAccFile;
        
        if (enable) {
		enable_accounting(AccFile);
	} else if (disable) {
		disable_accounting();
	} else if (uid != -1) {
           	accounting a(AccFile);
                a.show_gid(uid & 0xffff);
        } else if (gid != -1) {
           	accounting a(AccFile);
                a.show_gid(gid & 0xffff);
        } else if (user) {
		accounting a(AccFile);
		a.show_user(user);
	} else if (group) {
		accounting a(AccFile);
		a.show_group(group);
	} else if (all) {
		accounting a(AccFile);
		a.show_uid(-1);
	} else if (eoe_euid != -1) {
           	EoElog(eoe_euid);
        } else if (eoe_user) {
           	EoElogUser(eoe_user);
        }
                
        return 0;
}

void help_her()
{
   	cout<<"Wrong arguments. 'acct -?' for help.\n";
   	exit(1);
}

int put_banner()
{
	cout<<"acct v 0.9b -- BSD-accounting visu and EoE-logging tool.\n"
	      "(C) 1999/2000 by S. Krahmer\n"
	      " -- http://www.cs.uni-potsdam.de/homepages/students/linuxer --\n\n"
	      ;
   	return 0;
}

